﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Unsecure_Streaming
{
    public partial class MainWindow : Form
    {
        private enum HRESULT : uint
        {
            S_OK = 0,
            E_FAIL = 0x80004005,
            E_UNEXPECTED = 0x8000FFFF,
            E_NOTIMPL = 0x80004001,
            E_INVALIDARG = 0x80070057,
            IgnoreAndFixLater = 0xFFFFFFFF
        };

        private enum State
        {
            Disconnected,
            Connecting,
            Connected,
            Disconnecting
        }

        public MainWindow()
        {
            InitializeComponent();
        }

        private const int TRACK_ID = 1;

        private State m_state = State.Disconnected;
        private Bosch.VideoSDK.Device.DeviceConnector m_deviceConnector = new Bosch.VideoSDK.Device.DeviceConnector();
        private Bosch.VideoSDK.Device.DeviceProxy m_deviceProxy = null;
        private Bosch.VideoSDK.AxCameoLib.AxCameo m_axCameo = null;
        private Bosch.VideoSDK.CameoLib.Cameo m_cameo = null;

        private Bosch.VideoSDK.Live.VcaConfig m_vcaConfig = null;
        private Bosch.VideoSDK.MediaDatabase.Time64 m_startTime = null;
        private Bosch.VideoSDK.MediaDatabase.Time64 m_endTime = null;
        private Bosch.VideoSDK.MediaDatabase.SearchSession m_vcaEventSearch = null;
        private List<Bosch.VideoSDK.MediaDatabase.SearchResult> m_searchResults = new List<Bosch.VideoSDK.MediaDatabase.SearchResult>();
        private Bosch.VideoSDK.MediaDatabase.PlaybackController m_playbackController;
        private Bosch.VideoSDK.DataStream m_videoStream;
        //DataStream m_dataStream;

        private void MainWindow_Load(object sender, EventArgs e)
        {
            m_axCameo = new Bosch.VideoSDK.AxCameoLib.AxCameo();
            PanelCameo.Controls.Add(m_axCameo);
            m_axCameo.Dock = DockStyle.Fill;
            m_cameo = (Bosch.VideoSDK.CameoLib.Cameo)m_axCameo.GetOcx();
            m_cameo.ClickMode = Bosch.VideoSDK.CameoLib.ClickModeEnum.cmeNone;

            m_playbackController = new Bosch.VideoSDK.MediaDatabase.PlaybackController();

            m_deviceConnector.ConnectResult += new Bosch.VideoSDK.GCALib._IDeviceConnectorEvents_ConnectResultEventHandler(DeviceConnector_ConnectResult);
            ComboBoxProgID.SelectedIndex = 0;
            
            UpdateGUI();
        }

        private void MainWindow_FormClosing(object sender, FormClosingEventArgs e)
        {
            if ((m_state == State.Connecting) || (m_state == State.Disconnecting))
                e.Cancel = true;
        }

        private void MainWindow_FormClosed(object sender, FormClosedEventArgs e)
        {
            m_deviceConnector.ConnectResult -= new Bosch.VideoSDK.GCALib._IDeviceConnectorEvents_ConnectResultEventHandler(DeviceConnector_ConnectResult);
            if (m_state == State.Connected)
            {
                m_deviceProxy.ConnectionStateChanged -= new Bosch.VideoSDK.GCALib._IDeviceProxyEvents_ConnectionStateChangedEventHandler(DeviceProxy_ConnectionStateChanged);
                m_deviceProxy.Disconnect();
            }
            m_axCameo.SetVideoStream(null);
            m_axCameo.Dispose();
            

            if (m_playbackController != null)
            {
                for (int i = m_playbackController.MediaSessions.Count; i > 0; i--)
                {
                    m_playbackController.MediaSessions.Remove(m_playbackController.MediaSessions[i]);
                }
                m_playbackController = null;
            }

        }

        private void ButtonConnect_Click(object sender, EventArgs e)
        {
            if (m_state == State.Disconnected)
            {
                try
                {
                    m_state = State.Connecting;
                    m_deviceConnector.ConnectAsync(TextBoxUrl.Text, ComboBoxProgID.Text);
                }
                catch (Exception ex)
                {
                    CheckException(ex, "Failed to start asynchronous connection attempt to \"{0}\"", TextBoxUrl.Text);
                }
            }
            else if (m_state == State.Connected)
            {
                m_state = State.Disconnecting;
                m_deviceProxy.Disconnect();
            }

            UpdateGUI();
        }


        private void DeviceConnector_ConnectResult(Bosch.VideoSDK.Device.ConnectResultEnum connectResult, string url, Bosch.VideoSDK.Device.DeviceProxy deviceProxy)
        {
            bool success = false;

            if (connectResult == Bosch.VideoSDK.Device.ConnectResultEnum.creInitialized)
            {

                String infoString = "Device: "+url + " connected successfully";
                d_searchResults_listBx.Items.Add(infoString);


                if (deviceProxy.VideoInputs.Count > 0)
                {
                    success = true;

                    try
                    {
                        m_videoStream = deviceProxy.VideoInputs[1].Stream;
                        m_videoStream.Encoder = StreamcomboBox.SelectedIndex+1;
                        m_cameo.SetVideoStream(m_videoStream);
                    }
                    catch (Exception ex)
                    {
                        CheckException(ex, "Failed to render first video stream of {0}", url);
                        success = false;
                    }

                }
            }

            if (success)
            {
                m_deviceProxy = deviceProxy;
                m_deviceProxy.ConnectionStateChanged += new Bosch.VideoSDK.GCALib._IDeviceProxyEvents_ConnectionStateChangedEventHandler(DeviceProxy_ConnectionStateChanged);
                m_state = State.Connected;
                
            }
            else
            {
                if (deviceProxy != null)
                    deviceProxy.Disconnect();
                m_state = State.Disconnected;
                MessageBox.Show("Failed to connect to \"" + url + "\".");
            }


            UpdateGUI();
        }


        private void DeviceProxy_ConnectionStateChanged(object eventSource, Bosch.VideoSDK.Device.ConnectResultEnum state)
        {
            if (state == Bosch.VideoSDK.Device.ConnectResultEnum.creConnectionTerminated)
            {
                m_cameo.SetVideoStream(null);
                m_deviceProxy.ConnectionStateChanged -= new Bosch.VideoSDK.GCALib._IDeviceProxyEvents_ConnectionStateChangedEventHandler(DeviceProxy_ConnectionStateChanged);
                m_deviceProxy = null;
                m_state = State.Disconnected;

                UpdateGUI();

            }
        }

        private void TextBoxUrl_KeyDown(object sender, KeyEventArgs e)
        {
            if (((e.KeyCode == Keys.Return) || (e.KeyCode == Keys.Enter)) && ButtonConnect.Enabled)
                ButtonConnect_Click(this, EventArgs.Empty);
        }

        private void TextBoxUrl_TextChanged(object sender, EventArgs e)
        {
            UpdateGUI();
        }

        private void UpdateGUI()
        {
            if (m_state == State.Disconnected)
            {
                ButtonConnect.Text = "Connect";
                ButtonConnect.Enabled = (TextBoxUrl.Text.Length > 0);
            }
            else if (m_state == State.Connecting)
            {
                ButtonConnect.Text = "Connecting";
                ButtonConnect.Enabled = false;
            }
            else if (m_state == State.Connected)
            {
                ButtonConnect.Text = "Disconnect";
                ButtonConnect.Enabled = true;
            }
            else // if (m_state == State.Disconnecting)
            {
                ButtonConnect.Text = "Disconnecting";
                ButtonConnect.Enabled = false;
            }
        }

        private HRESULT CheckException(Exception ex, string format, params object[] args)
        {
            string message = string.Format(format, args) + ": " + ex.Message;
            if (ex.GetType() == typeof(System.Runtime.InteropServices.COMException))
            {
                uint errorCode = (uint)((System.Runtime.InteropServices.COMException)ex).ErrorCode;
                if (errorCode == (uint)HRESULT.E_FAIL)
                    return HRESULT.E_FAIL;
                else if (errorCode == (uint)HRESULT.E_UNEXPECTED)
                    return HRESULT.E_UNEXPECTED;
            }
            else if (ex.GetType() == typeof(System.NotImplementedException))
                return HRESULT.E_NOTIMPL;
            else if (ex.GetType() == typeof(System.ArgumentException))
                return HRESULT.E_INVALIDARG;

            if (MessageBox.Show(message + "\n\nTerminate application?", "Unexpected Exception", MessageBoxButtons.YesNo, MessageBoxIcon.Error) == DialogResult.Yes)
            {
                System.Diagnostics.Process.GetCurrentProcess().Kill();
                throw ex;
            }
            else
                return HRESULT.IgnoreAndFixLater;
        }

        private void createARuleButton_Click(object sender, EventArgs e)
        {

            if (m_vcaConfig != null)
            {
                m_vcaConfig.StateChanged -= new Bosch.VideoSDK.GCALib._IVcaConfigEvents_StateChangedEventHandler(m_vcaConfig_StateChanged);
                m_vcaConfig = null;
            }

            m_vcaConfig = m_axCameo.VcaConfig;
            m_vcaConfig.Initialize();
            m_vcaConfig.StateChanged +=new Bosch.VideoSDK.GCALib._IVcaConfigEvents_StateChangedEventHandler(m_vcaConfig_StateChanged);
        }

        void  m_vcaConfig_StateChanged(Bosch.VideoSDK.Live.VcaConfig EventSource, Bosch.VideoSDK.Live.VcaConfigStates State)
        {
            switch (State)
            {
                case Bosch.VideoSDK.Live.VcaConfigStates.vcsInitialized:

                    m_vcaConfig.AttachPlugin(m_vcaConfig.ViprocId, 0, 2, "en");
                    break;

                case Bosch.VideoSDK.Live.VcaConfigStates.vcsAttached:
                    m_vcaConfig.DisplayMode = Bosch.VideoSDK.Live.VcaDisplayModes.vcmEnableEditing;
                    m_vcaConfig.DisplayMode = Bosch.VideoSDK.Live.VcaDisplayModes.vcmConfigureAndRenderVCD;
                    m_vcaConfig.StateChanged -= new Bosch.VideoSDK.GCALib._IVcaConfigEvents_StateChangedEventHandler(m_vcaConfig_StateChanged);
                    break;

            }

        }


        #region event search
        private void d_vcaEventSearch_btn_Click(object sender, EventArgs e)
        {
            startVCAEventSearch();
            d_searchResults_listBx.Items.Clear();
            m_searchResults.Clear();
        }


        private void startVCAEventSearch()
        {
                        m_startTime = new Bosch.VideoSDK.MediaDatabase.Time64();
            m_endTime = new Bosch.VideoSDK.MediaDatabase.Time64();
                       
            m_vcaEventSearch = m_deviceProxy.MediaDatabase.CreateSearchSession(Bosch.VideoSDK.MediaDatabase.SearchTypeEnum.steVca);
            m_vcaEventSearch.ResultAvailable+= new Bosch.VideoSDK.GCALib._ISearchSessionEvents_ResultAvailableEventHandler(m_vcaEventSearch_ResultAvailable);
            m_vcaEventSearch.Progress += new Bosch.VideoSDK.GCALib._ISearchSessionEvents_ProgressEventHandler(vcaSearch_Progress);
            m_vcaEventSearch.StartTimeFilter = m_startTime;
            m_vcaEventSearch.EndTimeFilter = m_endTime;
            m_vcaEventSearch.AddIdentifierFilter(TRACK_ID);

            m_vcaEventSearch.VcaConfigDataFilter = m_axCameo.VcaConfig.Configuration;
            m_vcaEventSearch.Start(0);
        }

        void m_vcaEventSearch_ResultAvailable(Bosch.VideoSDK.MediaDatabase.SearchTypeEnum Type, Bosch.VideoSDK.MediaDatabase.SearchResult pResult, Bosch.VideoSDK.MediaDatabase.SearchSession pSearchSession)
        {
            m_searchResults.Add(pResult);
            String infoString = "Event type: " + pResult.EventType + " Rule: " + pResult.Text + " Start time : " + pResult.StartTime.UTC + " End time: " + pResult.EndTime.UTC;
            d_searchResults_listBx.Items.Add(infoString);
        }


        void vcaSearch_Progress(int Progress, Bosch.VideoSDK.MediaDatabase.SearchSession pSearchSession)
        {
            if (Progress == 100 && pSearchSession.IsComplete)
            {
                String infoString = "Search completed : " + Progress + "%";
                d_searchResults_listBx.Items.Add(infoString);
            }
        }
        #endregion

        private void d_searchResults_listBx_SelectedIndexChanged(object sender, EventArgs e)
        {
            int selectedIndex = d_searchResults_listBx.SelectedIndex;

            m_videoStream = m_deviceProxy.MediaDatabase.GetMediaSession(1, m_playbackController).GetVideoStream();
            m_cameo.SetVideoStream(m_videoStream);
            m_playbackController.Seek(m_searchResults[selectedIndex].StartTime);
            m_playbackController.Play(100);

        }

        private void groupBox1_Enter(object sender, EventArgs e)
        {

        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {

        }
    }
}
